"""
This module contains the API for the game.
"""

from pygame import init  # pylint: disable=no-name-in-module,import-error

from .generators import *
from .events import *
from .utils import *
from .random import *

init()
